; Rafael compiler service routines
; Part of RCOMP.CPP, called from C++ level

; Compiler dynamic database initialization
; Creates an empty parameter list which is bound
; to the "parlist" variable. The format of this
; list is the following :
; ( (<nodenum1> <parlist1>)
;   (<nodenum2> <parlist2>)
;	      ...
;   (<nodenumn> <parlistn> )
;				)
;where nodenum is the number of the node and parlist
;is the parameter list associated with it.
(defun init-comp()
	(setq parlist '() )
	(setq procnum 0)
        (init-lib)
)

; Called when the compiler is about to terminate.
; Clears the parameter list database
(defun terminate-comp()
	(setq parlist '() )
)

; Returns the number of the elements of its parameter list
(defun elem-no (parlist)
        (do*  ((n 0))
              ((null parlist) n)
              (setq n (+ n 1))
              (setq parlist (cdr parlist)))
)

; Transforms the parameter list so that a 'quote' is
; inserted before each parameter list element
(defun transform-parlist(plist)
  (let (vlist)
    (dolist (n plist vlist)
        (if (eql vlist nil)
            (setq vlist (list (list 'quote n)))
            (setq vlist (append vlist (list (list 'quote n))))
        )
    )
  )  
)
   
; Gets the number of the input and output connections of
; the operator. Calling : (get-io <opname>),Returns :
; (i o) where i is the number of input, o is the number
; of output connections
(defun get-io (op-header)
    (list (elem-no (second op-header))
          (elem-no (third op-header)))
)

; Adds a parameter list to the dynamic database            
; Calling form : (add-parlist <operator name> <nodenum> <parlist>)
(defun add-parlist(operator nodenum plis)
    (setq parlist
        (append (list (list nodenum plis)) parlist))
    nil
)

; Returns the parameter list of the given node
; Gets the node number
(defun search-parlist(nodenum)
        (setq plist parlist)
        (do*  ()
              ((or (null plist) (eql nodenum (caar plist)))
                         (second (car plist)))
              (setq plist (cdr plist)))
)

; Returns the size of an input node
; Calling form : (get-isize <operator name> <input no> <nodenum>)
(defun get-isize(operator inum node)
  (setq plist (transform-parlist (search-parlist node)))
  (setq pprot (nth 4 operator))
  (setq inplist (second operator))
  (setq sexpr
    (do* ((n 1))
       ((= n inum) (cadar inplist))
       (setq inplist (cdr inplist))
       (setq n (+ n 1))
    )
  )
  (eval (append (list (list 'lambda pprot sexpr)) plist))
)

; Returns the size of an output node and the allocation factor
; Calling form : (get-osize <operator name> <output no> <nodenum>)
(defun get-osize(operator inum node)
  (setq plist (transform-parlist (search-parlist node)))
  (setq pprot (nth 4 operator))
  (setq outlist (third operator))
  (setq sexpr
    (do* ((n 1))
       ((= n inum) (cdar outlist))
       (setq outlist (cdr outlist))
       (setq n (+ n 1))
    )
  )
  (list
    (eval (append (list (list 'lambda pprot (car sexpr))) plist)) 
    (second sexpr)
  )
)

; Returns the interpolation/decimation factors of the node
; Calling form : (get-IPF <operator name> <output no> <nodenum>)
(defun get-ipf(operator inum node)
  (setq plist (transform-parlist (search-parlist node)))
  (setq pprot (nth 4 operator))
  (setq outlist (third operator))
  (setq sexpr
    (do* ((n 1))
       ((= n inum) (cdar outlist))
       (setq outlist (cdr outlist))
       (setq n (+ n 1))
    )
  )
  (list
    (eval (append (list (list 'lambda pprot (third sexpr))) plist)) 
    (eval (append (list (list 'lambda pprot (fourth sexpr))) plist)) 
  )
)

; Returns the type list of the operator
; Return list ((itype1 ... itypen)(otype1 ... otypen))
; Calling form : (get-otype <op name>)
(defun get-optype(operator)
  (let (inplist outlist)
    (list
      (dolist (n (second operator) inplist)
        (setq inplist (append inplist (list (car n))))
      ) 
      (dolist (n (third operator) outlist)
        (setq outlist (append outlist (list (car n))))
      )
    )
  )
)

; Returns the execution time of a node
; Calling form : (get-exectime <operator> <device> <nodenum>)
(defun get-exectime(operator device node)
  (setq plist (transform-parlist (search-parlist node)))
  (setq pprot (nth 4 operator))
  (setq execlist (fourth operator))
  (setq sexpr
    (do* ()
       ((or (eql (caar execlist) device)
            (null execlist))
                 (cadar execlist))
       (setq execlist (cdr execlist))
    )
  )
  (eval
    (append (list (list 'lambda pprot sexpr)) plist)
  ) 
)

; Returns the devices and execution times for a given
; operator on a given node.
; Calling form : (get-exectimes <operator> <nodenum>)
; Returns : the list of the devices which support this
; operator and the execution times on it in the form :
; (<device1> <time1> ... <devicen> <timen>)
(defun get-exectimes(operator node)
  (let (sexpr texpr retlist acttime)
    (setq plist (transform-parlist (search-parlist node)))
    (setq pprot (nth 4 operator))
    (setq devicelist (fourth operator))
    (dolist (sexpr devicelist)
      (setq texpr (second sexpr))
      (setq retlist (append retlist (list (first sexpr))))
      (setq acttime (eval
        (append (list (list 'lambda pprot texpr)) plist))
      )
      (setq retlist (append retlist (list acttime)))
    )                   ;dolist
    retlist
  )                     ;let
)

; Returns the time expression for the given operator
(defun search-time-expr(operator device)
  (let ((tlist (fourth operator)))
    (do* ()
      ((or (eql (caar tlist) device)
           (null tlist))
       (cadar tlist)
      )
      (setq tlist (cdr tlist))  
    )
  )
)

; Returns the best available candidate processor for the
; given node. The best candidate is the processor on which
; the operator achieves the shortest running time 
; Calling form : (get-candidate <operator> <device list> <nodenum>)
; Returns : the candidate device and the execution time on it
(defun get-candidate(operator devicelist node)
  (let (bestdev besttime acttime)
    (setq plist (transform-parlist (search-parlist node)))
    (setq pprot (nth 4 operator))
    (setq execlist (fourth operator))
    (dolist (device devicelist)
      (if (not (null
               (setq sexpr (search-time-expr operator device))
               )
          )
        (progn
          (setq acttime (eval
            (append (list (list 'lambda pprot sexpr)) plist))
          )
	  (if (or (null besttime) (< acttime besttime) )
            (progn
	      (setq besttime acttime)
              (setq bestdev device)
            )           ;progn
          )             ;if
        )               ;progn
      )                 ;if 
    )                   ;dolist 
    (list bestdev besttime) 
  )
)

;Slices down about 500 characters from outtext and returns it.
;The unprocessed part remains in outtext
(defun return-code()
    (if (< (length outtext) 500)
        (prog1
          (setq retval outtext)
          (setq outtext "")
        )
        (prog1
          (setq retval (subseq outtext 0 500))
          (setq outtext (subseq outtext 500))
          (setq n (search "\n" outtext))
          (setq retval (concatenate 'STRING retval (subseq outtext 0 (+ n 1))))
          (setq outtext (subseq outtext (+ 1 n)))
        )
    )
    retval
)

; Executes the input strategy function of the given node
; for the given operator. Gets the operator name, the
; device ID, the node number, the list of the input and
; output variables The input strategy function returns the
; generated output text in outtext.
(defun get-input-code(operator device node inputs outputs )
  (setq plist (search-parlist node))
  (setq pprot (nth 4 operator))
  (setq execlist (first operator))
  (setq sexpr
    (do* ()
       ((or (eql (caar execlist) device)
            (null execlist))
                 (second (car execlist)))
       (setq execlist (cdr execlist))
    )
  )
  (if sexpr
    (eval (list sexpr (list 'quote plist) (list 'quote inputs)
              (list 'quote outputs) (list 'quote node) ))
    (setq outtext "")
  )
  nil
)

; Executes the output strategy function of the given node
; for the given operator. Gets the operator name, the
; device ID, the node number, the list of the input and
; output variables The output strategy function returns the
; generated output text in outtext.
(defun get-output-code(operator device node inputs outputs )
  (setq plist (search-parlist node))
  (setq pprot (nth 4 operator))
  (setq execlist (first operator))
  (setq sexpr
    (do* ()
       ((or (eql (caar execlist) device)
            (null execlist))
                 (third (car execlist)))
       (setq execlist (cdr execlist))
    )
  )
  (if sexpr
    (eval (list sexpr (list 'quote plist) (list 'quote inputs)
              (list 'quote outputs) node ))
    (setq outtext "")
  ) 
  nil
)

; Executes the constructor function of the given node for
; the given operator. Gets the operator name, the device
; ID, the node number, the list of the input and output
; variables. The constructor function returns the generated
; text in outtext variable. Returns NIL if no constructor
; function exists for the operator, 1 else.
(defun get-constr(operator device node inputs outputs )
  (setq plist (search-parlist node))
  (setq pprot (nth 4 operator))
  (setq execlist (nth 5 operator))
  (setq sexpr
    (do* ()
       ((or (eql (caar execlist) device)
            (null execlist))
                 (cadar execlist))
       (setq execlist (cdr execlist))
    )
  )
  (if (null sexpr)
      nil
      (eval (list sexpr (list 'quote plist)
              (list 'quote inputs) (list 'quote outputs) 
              (list 'quote node) ))
  )
)

; Executes the destructor function of the given node for
; the given operator. Gets the operator name, the device
; ID, the node number, the list of the input and output
; variables. The constructor function returns the generated
; text in outtext variable. Returns NIL if no destructor
; function exists for the operator, 1 else.
(defun get-destr(operator device node inputs outputs )
  (setq plist (search-parlist node))
  (setq pprot (nth 4 operator))
  (setq execlist (nth 6 operator))
  (setq sexpr
    (do* ()
       ((or (eql (caar execlist) device)
            (null execlist))
                 (cadar execlist))
       (setq execlist (cdr execlist))
    )
  )
  (if (null sexpr)
      nil
      (eval (list sexpr (list 'quote plist)
              (list 'quote inputs) (list 'quote outputs)
              (list 'quote node) ))
  )
)

; Returns the actual element of the processor list and
; steps to the next list element. This function is
; used by the compiler core to fetch the processor list
; Returns NIL if end of the proc list.
(defun get-nextproc()
  (prog1
    (nth procnum target)
    (setq procnum (+ procnum 1))
  )
)

