/****** robject.h
		Rafael object library - header
		   Written by Gabor Paller
					********/

// Node number type
#define	NODETYPE	unsigned int
// Input number type
#define	INPUTTYPE	unsigned int
// Not valid entry
#define	NOT_VALID	0xFFFF

// ListObject
class ListObject
{
public:
  ListObject	*BackLink;		// Backward chain
  ListObject	*ForwLink;		// Forward chain
  ListObject	*SubList;
  ListObject	*ListHead;
  NODETYPE	NodeNum;

  ListObject( NODETYPE );		  // Constructor
  void Insert( ListObject * );	  // Insert into sublist
  void InsertFromTail( ListObject *, unsigned );
  void Remove( ListObject * );
  ListObject *Search( NODETYPE ); // Search given node
};

typedef ListObject *PtListObject;

// Chunk object

#define	CHK_NOMEM	0xFFFFFFFFL

class Chunk : public ListObject
{
public:
  unsigned long	ID;		// Variable ID
  unsigned long	StartAddr;	// Start address
  unsigned long	Size;		// Size
  unsigned long	Avail;		// Space available in the chunk

  Chunk( int, unsigned long,unsigned long,unsigned long );
  Chunk();
  unsigned long Alloc( unsigned long );
  void Insert( Chunk * );
  void Remove( Chunk * );
};

typedef	Chunk *PtChunk;

// Variable object

class Variable : public ListObject
{
public:
  unsigned long	ID;		// Variable ID
  unsigned long	Address;	// Address to allocate
  unsigned long	Size;		// Size of the variable
  unsigned	Consumed;	// 0 if goes out of scope
  PtChunk	AssocChunk;	// Associated chunk in the memory map
  PtListObject	GraphPtr;	// The node which this variable belongs to
  PtListObject	ProcPtr;	// Processor descriptor of
				// the processor on which
				// the variable was allocated
  PtListObject	BlockPtr;	// Points to the block on
				// which the variable was allocated
  Variable	*VarLink;	// Links the variable instances
				// on different processors
				// circular list !
  Variable();
  Variable( int, unsigned long );
  void Insert( Variable * );
};

typedef Variable *PtVariable;

// Channel object

class Channel : public ListObject
{
public:
// Occupation of the channel by the last send operation
  unsigned long		SendStartOccupied;
  unsigned long		SendEndOccupied;
// Occupation of the channel by the last receive operation
  unsigned long		RecStartOccupied;
  unsigned long		RecEndOccupied;

  Channel( NODETYPE );
  Channel();
  void Insert( Channel * );
  Channel *Search( NODETYPE );
};

typedef Channel *PtChannel;

// Execution time object

class ExecTimeList : public ListObject
{
public:
  unsigned long	ExecTime;
  char	Dev[32];

  ExecTimeList( char[],unsigned long );
  ExecTimeList *SearchDevice( char[] );
  void Insert( ExecTimeList * );
  void ClearSubList();
};

typedef ExecTimeList *PtExecTimeList;

// Node object

class NodeObject : public ListObject
{
public:
  NodeObject	*ForwNode;	// List of nodes belonging
  NodeObject	*BackNode;	// to one operator
  NodeObject	*UpNode;	// Parent node
  PtListObject	InfoNode;	// Nodeinfo ptr
  BOOL		Ready;		// Node ready
  BOOL		Auto;		// TRUE if the output variable is auto
				// allocated
  BOOL		Blocked;	// Multi-purpose variable
  INPUTTYPE	InputNum;	// Number of input
  INPUTTYPE	OutputNum;	// Number of output
  PtVariable	NodeVariable;	// Variable associated with the output
  unsigned long	Size;		// Size of the node
  unsigned long	Aux;		// Aux var
  char		TypeName[32];	// Type of the node
  unsigned long	VarReadyTime;	// Time when node gets ready
  unsigned	UIPF;		// Upscale factor of the output
  unsigned	DIPF;		// Downscale factor of the output
  unsigned	UpScale;	// Interpolation factor
  unsigned	DownScale;	// Decimation factor

  NodeObject( NODETYPE,INPUTTYPE ,INPUTTYPE );
  void Insert( NodeObject * );
  void InsertOp( NodeObject * );
  virtual BOOL IsReady();
  void ResetFlags();
  NodeObject *Search( INPUTTYPE, INPUTTYPE );
  void SubstituteTypeName( LPSTR,LPSTR );
};

typedef NodeObject *PtNodeObject;

// DelayInput object - node object with for delay inputs

class DelayInputObject : public NodeObject
{
public:
  DelayInputObject( NODETYPE,INPUTTYPE ,INPUTTYPE );
  virtual BOOL IsReady();
};

typedef DelayInputObject *PtDelayInputObject;

// DelayOutput object - node object with for delay outputs

class DelayOutputObject : public NodeObject
{
public:
  DelayOutputObject( NODETYPE,INPUTTYPE ,INPUTTYPE );
  virtual BOOL IsReady();
};

typedef DelayOutputObject *PtDelayOutputObject;

// Operator types - type masks
#define	OPTYPE_PROBE	1       // Probe
#define	OPTYPE_OP	2	// Operator
#define OPTYPE_ODELAY	4	// Delay output
#define	OPTYPE_IDELAY	8	// Delay input
#define OPTYPE_BLOCK	16	// Block

#define OPTYPE_ALL	0xFFFF	// Mask all types
#define OPTYPE_OUT	(OPTYPE_PROBE | OPTYPE_OP | OPTYPE_ODELAY | OPTYPE_BLOCK)
#define OPTYPE_IN	(OPTYPE_PROBE | OPTYPE_OP | OPTYPE_IDELAY | OPTYPE_BLOCK)

// NodeInfo object
class NodeInfo : public ListObject
{
public:
  unsigned	Inputs;		// Number of inputs
  unsigned	Outputs;	// Number of outputs
  int		Type;	     	// Node type (probe or operator)
  char		Operator[32];	// Operator name
  PtNodeObject	GraphPtr;	// Pointer to the graph element
  BOOL		Executed;	// Executed flag
  BOOL		Visited;	// Visited by IPF checker
  unsigned long	ASAP;		// Asap time
  unsigned long ALAP;		// Alap time
  unsigned long	ExecTime;	// Execution time
  unsigned long	Aux;		// Auxiliary variable
  unsigned long	StartTime;	// Operator start time
  unsigned long	FinishTime;	// Operator finish time
  NodeInfo	*Alias;		// Points to the original
				// nodeinfo if alias
  ListObject	*SchedProc;	// Type : PtProcessor !
				// Points to the processor
				// descriptor to which the
				// operator was assigned

  NodeInfo();
  NodeInfo( NODETYPE,int, int ,int ,LPSTR );
  void Insert( NodeInfo * );
  NodeInfo *Search( NODETYPE, int );
  void CalculateStartTime();
  BOOL GetScales( unsigned *, unsigned *);
};

typedef NodeInfo *PtNodeInfo;

// Activity list object
#define	ACT_OPERATOR	1
#define	ACT_SEND	2
#define	ACT_RECEIVE	3
#define	ACT_GETDELAY	4
#define	ACT_PUTDELAY	5
#define	ACT_INPPROBE	6
#define	ACT_OUTPROBE	7
#define	ACT_SAMPLERATE	8
#define	ACT_BLOCK	9

class Activity : public ListObject
{
public:
  int		ActId;		// Activity ID
  unsigned long	STime;		// Scheduling time
  unsigned long	CommInProgress;	// Scheduling time for the background receive
  unsigned long	ExecTime;	// Execution time
  PtVariable	VarField;	// Associated variable if
				// comm activity
  int		Aux;		// Auxiliary data field
				// partner proc number if
				// if comm activity
  Activity	*PartnerAct;	// Receiver activity if
				// comm
  unsigned	UpScale;	// Upsample factor
  unsigned	DownScale;	// Downsample factor

  Activity();
  Activity( NODETYPE,int ,unsigned long,int );
  void Insert( Activity * );
  unsigned long MaxTime();
};

typedef Activity *PtActivity;

// Processor list object

class Processor : public ListObject
{
public:
  unsigned long	Memory;		// Memory available
  unsigned long	OccupiedTime;	// Time till the processor
				// is occupied
  unsigned long	Aux;		// Aux value (used to store
				// the comm cost.
  char	Device[32];		// Device name

  unsigned	UpScale;	// Upsample and downsample
  unsigned	DownScale;	// factors
  Processor( NODETYPE, unsigned long, char[] );
  void Insert( Processor * );
};

typedef	Processor *PtProcessor;

#define	PTRANS_INPUT	1
#define	PTRANS_OUTPUT	2

// Parameter translation object
class PTrans : public ListObject
{
public:
  int	Dir;
  NODETYPE PNodeNum;
  NODETYPE PIONum;
  char	Type[32];

  PTrans();
  PTrans( NODETYPE,NODETYPE,NODETYPE,int,char[] );
  void Insert( PTrans * );
  PTrans *Search( NODETYPE,int );
};

typedef PTrans *PtPTrans;

// Block list object
class Block : public ListObject
{
public:
  char		BlockName[32];
  unsigned long	ExecTime;
  int		InputNum,OutputNum;
  BOOL		IsRoot;
  BOOL		Scheduled;
  BOOL		ConstrGenerated;
  BOOL		OpGenerated;
  BOOL		DestrGenerated;

// Parameter translation list head
  PTrans	PTList;
// Dictionary of nodes
  NodeInfo	NodeList;
// Activity list. On first level activities denote
// processors (processor number stored in 'Node'
// field), on the second level the activites are
// inserted by ascending schedule time.
  Activity	ActivityList;

// Channel list. This list contains the descriptors
// of the system communication channels where the
// state of the channels is administrated
  Channel		Channels;

// Variable list. This list stores the variables local
// to the block
  Variable		VariableList;

// Chunk list. This list stores the memory chunks local
// to the block
  Chunk			ChunkList;

  Block( char[],BOOL );
  void Insert( Block * );
  Block *SearchBlockName( char[] );
};

typedef Block *PtBlock;

/******************** END HEADER - ROBJECT.H ***************/
